# -*- coding: utf-8 -*-
# Copyright (C) 2018 - 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Module for connections to GVM server daemons like gvmd and ospd.
"""
import logging
import socket as socketlib
import ssl
import time

from typing import Optional, Union

import paramiko

from lxml import etree

from gvm.errors import GvmError


logger = logging.getLogger(__name__)

BUF_SIZE = 16 * 1024
DEFAULT_READ_TIMEOUT = 60  # in seconds
DEFAULT_TIMEOUT = 60  # in seconds
DEFAULT_GVM_PORT = 9390
DEFAULT_UNIX_SOCKET_PATH = "/var/run/gvmd.sock"
DEFAULT_SSH_PORT = 22
DEFAULT_HOSTNAME = '127.0.0.1'
MAX_SSH_DATA_LENGTH = 4095


class XmlReader:
    """
    Read a XML command until its closing element
    """

    def _start_xml(self):
        self._first_element = None
        # act on start and end element events and
        # allow huge text data (for report content)
        self._parser = etree.XMLPullParser(
            events=("start", "end"), huge_tree=True
        )

    def _is_end_xml(self):
        for action, obj in self._parser.read_events():
            if not self._first_element and action in "start":
                self._first_element = obj.tag

            if (
                self._first_element
                and action in "end"
                and str(self._first_element) == str(obj.tag)
            ):
                return True
        return False

    def _feed_xml(self, data):
        try:
            self._parser.feed(data)
        except etree.ParseError as e:
            raise GvmError(
                "Cannot parse XML response. Response data "
                "read {0}".format(data),
                e,
            ) from None


class GvmConnection(XmlReader):
    """
    Base class for establishing a connection to a remote server daemon.

    Arguments:
        timeout: Timeout in seconds for the connection. None to
            wait indefinitely
    """

    def __init__(self, timeout: Optional[int] = DEFAULT_TIMEOUT):
        self._socket = None
        self._timeout = timeout

    def _read(self):
        return self._socket.recv(BUF_SIZE)

    def connect(self):
        """Establish a connection to a remote server"""
        raise NotImplementedError

    def send(self, data: Union[bytes, str]):
        """Send data to the connected remote server

        Arguments:
            data: Data to be send to the server. Either utf-8 encoded string or
                bytes.
        """
        if self._socket is None:
            raise GvmError("Socket is not connected")

        if isinstance(data, str):
            self._socket.sendall(data.encode())
        else:
            self._socket.sendall(data)

    def read(self) -> str:
        """Read data from the remote server

        Returns:
            str: data as utf-8 encoded string
        """
        response = ""

        self._start_xml()

        if self._timeout is not None:
            now = time.time()

            break_timeout = now + self._timeout

        while True:
            data = self._read()

            if not data:
                # Connection was closed by server
                raise GvmError("Remote closed the connection")

            self._feed_xml(data)

            response += data.decode("utf-8", errors="ignore")

            if self._is_end_xml():
                break

            if self._timeout is not None:
                now = time.time()

                if now > break_timeout:
                    raise GvmError("Timeout while reading the response")

        return response

    def disconnect(self):
        """Disconnect and close the connection to the remote server"""
        try:
            if self._socket is not None:
                self._socket.close()
        except OSError as e:
            logger.debug("Connection closing error: %s", e)

    def finish_send(self):
        """Indicate to the remote server you are done with sending data"""
        # shutdown socket for sending. only allow reading data afterwards
        self._socket.shutdown(socketlib.SHUT_WR)


class SSHConnection(GvmConnection):
    """
    SSH Class to connect, read and write from GVM via SSH

    Arguments:
        timeout: Timeout in seconds for the connection.
        hostname: DNS name or IP address of the remote server. Default is
            127.0.0.1.
        port: Port of the remote SSH server. Default is port 22.
        username: Username to use for SSH login. Default is "gmp".
        password: Passwort to use for SSH login. Default is "".
    """

    def __init__(
        self,
        *,
        timeout: Optional[int] = DEFAULT_TIMEOUT,
        hostname: Optional[str] = DEFAULT_HOSTNAME,
        port: Optional[int] = DEFAULT_SSH_PORT,
        username: Optional[str] = "gmp",
        password: Optional[str] = ""
    ):
        super().__init__(timeout=timeout)

        self.hostname = hostname
        self.port = int(port)
        self.username = username
        self.password = password

    def _send_all(self, data):
        while data:
            sent = self._stdin.channel.send(data)

            if not sent:
                # Connection was closed by server
                raise GvmError("Remote closed the connection")

            data = data[sent:]

    def connect(self):
        """
        Connect to the SSH server and authenticate to it
        """
        self._socket = paramiko.SSHClient()
        self._socket.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        try:
            self._socket.connect(
                hostname=self.hostname,
                username=self.username,
                password=self.password,
                timeout=self._timeout,
                port=int(self.port),
                allow_agent=False,
                look_for_keys=False,
            )
            self._stdin, self._stdout, self._stderr = self._socket.exec_command(
                "", get_pty=False
            )

        except (
            paramiko.BadHostKeyException,
            paramiko.AuthenticationException,
            paramiko.SSHException,
        ) as e:
            raise GvmError("SSH Connection failed", e) from None

    def _read(self):
        return self._stdout.channel.recv(BUF_SIZE)

    def send(self, data: Union[bytes, str]):
        self._send_all(data)

    def finish_send(self):
        # shutdown socket for sending. only allow reading data afterwards
        self._stdout.channel.shutdown(socketlib.SHUT_WR)


class TLSConnection(GvmConnection):
    """
    TLS class to connect, read and write from a remote GVM daemon via TLS
    secured socket.

    Arguments:
        timeout: Timeout in seconds for the connection.
        hostname: DNS name or IP address of the remote TLS server.
        port: Port for the TLS connection. Default is 9390.
        certfile: Path to PEM encoded certificate file. See
            `python certificates`_ for details.
        cafile: Path to PEM encoded CA file. See `python certificates`_
            for details.
        keyfile: Path to PEM encoded private key. See `python certificates`_
            for details.
        password: Password for the private key. If the password argument is not
            specified and a password is required it will be interactively prompt
            the user for a password.

    .. _python certificates:
        https://docs.python.org/3/library/ssl.html#certificates
    """

    def __init__(
        self,
        *,
        certfile: Optional[str] = None,
        cafile: Optional[str] = None,
        keyfile: Optional[str] = None,
        hostname: Optional[str] = DEFAULT_HOSTNAME,
        port: Optional[int] = DEFAULT_GVM_PORT,
        password: Optional[str] = None,
        timeout: Optional[int] = DEFAULT_TIMEOUT
    ):
        super().__init__(timeout=timeout)

        self.hostname = hostname
        self.port = port
        self.certfile = certfile
        self.cafile = cafile
        self.keyfile = keyfile
        self.password = password

    def _new_socket(self):
        transport_socket = socketlib.socket(
            socketlib.AF_INET, socketlib.SOCK_STREAM
        )

        if self.certfile and self.cafile and self.keyfile:
            context = ssl.create_default_context(
                ssl.Purpose.SERVER_AUTH, cafile=self.cafile
            )
            context.check_hostname = False
            context.load_cert_chain(
                certfile=self.certfile,
                keyfile=self.keyfile,
                password=self.password,
            )
            sock = context.wrap_socket(transport_socket, server_side=False)
        else:
            context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
            sock = context.wrap_socket(transport_socket)

        sock.settimeout(self._timeout)

        return sock

    def connect(self):
        self._socket = self._new_socket()
        self._socket.connect((self.hostname, int(self.port)))


class UnixSocketConnection(GvmConnection):
    """
    UNIX-Socket class to connect, read, write from a GVM server daemon via
    direct communicating UNIX-Socket

    Arguments:
        path: Path to the socket. Default is "/var/run/gvmd.sock".
        timeout: Timeout in seconds for the connection. Default is 60 seconds.
    """

    def __init__(
        self,
        *,
        path: Optional[str] = DEFAULT_UNIX_SOCKET_PATH,
        timeout: Optional[int] = DEFAULT_TIMEOUT
    ):
        super().__init__(timeout=timeout)

        self.path = path

    def connect(self):
        """Connect to the UNIX socket"""
        self._socket = socketlib.socket(
            socketlib.AF_UNIX, socketlib.SOCK_STREAM
        )
        self._socket.settimeout(self._timeout)
        try:
            self._socket.connect(self.path)
        except FileNotFoundError:
            raise GvmError(
                "Socket {path} does not exist".format(path=self.path)
            ) from None
        except ConnectionError:
            raise GvmError(
                "Could not connect to socket {}".format(self.path)
            ) from None


class DebugConnection:
    """Wrapper around a connection for debugging purposes

    Allows to debug the connection flow including send and read data. Internally
    it uses the python `logging`_ framework to create debug messages. Please
    take a look at `the logging tutorial
    <https://docs.python.org/3/howto/logging.html#logging-basic-tutorial>`_
    for further details.

    Usage example:

    .. code-block:: python

        import logging

        logging.basicConfig(level=logging.DEBUG)

        socketconnection = UnixSocketConnection(path='/var/run/gvm.sock')
        connection = DebugConnection(socketconnection)
        gmp = Gmp(connection=connection)

    Arguments:
        connection: GvmConnection to observe

    .. _logging:
        https://docs.python.org/3/library/logging.html
    """

    def __init__(self, connection: GvmConnection):
        self._connection = connection

    def read(self) -> str:
        data = self._connection.read()

        logger.debug("Read %s characters. Data %s", len(data), data)

        self.last_read_data = data
        return data

    def send(self, data):
        self.last_send_data = data

        logger.debug("Sending %s characters. Data %s", len(data), data)

        return self._connection.send(data)

    def connect(self):
        logger.debug("Connecting")

        return self._connection.connect()

    def disconnect(self):
        logger.debug("Disconnecting")

        return self._connection.disconnect()

    def finish_send(self):
        logger.debug("Finish send")

        self._connection.finish_send()



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from decimal import Decimal

from gvm.errors import RequiredArgument, InvalidArgumentType

from gvm.protocols.gmpv7 import SeverityLevel


class GmpCreateNoteTestCase:
    def test_create_note(self):
        self.gmp.create_note('foo', nvt_oid='oid1')

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '</create_note>'
        )

    def test_create_note_missing_text(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_note(None, 'od1')

        with self.assertRaises(RequiredArgument):
            self.gmp.create_note('', 'oid1')

    def test_create_note_missing_nvt_iod(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_note('foo', None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_note('foo', '')

    def test_create_note_with_hosts(self):
        self.gmp.create_note('foo', nvt_oid='oid1', hosts=[])

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', hosts=['h1', 'h2'])

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<hosts>h1,h2</hosts>'
            '</create_note>'
        )

    def test_create_note_with_port(self):
        self.gmp.create_note('foo', nvt_oid='oid1', port='666')

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<port>666</port>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', port=666)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<port>666</port>'
            '</create_note>'
        )

    def test_create_note_with_result_id(self):
        self.gmp.create_note('foo', nvt_oid='oid1', result_id='r1')

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<result id="r1"/>'
            '</create_note>'
        )

    def test_create_note_with_task_id(self):
        self.gmp.create_note('foo', nvt_oid='oid1', task_id='t1')

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<task id="t1"/>'
            '</create_note>'
        )

    def test_create_note_with_severity(self):
        self.gmp.create_note('foo', nvt_oid='oid1', severity='5.5')

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<severity>5.5</severity>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', severity=5.5)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<severity>5.5</severity>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', severity=Decimal(5.5))

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<severity>5.5</severity>'
            '</create_note>'
        )

    def test_create_note_with_threat(self):
        self.gmp.create_note('foo', nvt_oid='oid1', threat=SeverityLevel.HIGH)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<threat>High</threat>'
            '</create_note>'
        )

    def test_create_note_invalid_threat(self):
        with self.assertRaises(InvalidArgumentType):
            self.gmp.create_note('foo', nvt_oid='oid1', threat='')

        with self.assertRaises(InvalidArgumentType):
            self.gmp.create_note('foo', nvt_oid='oid1', threat='foo')

    def test_create_note_with_seconds_active(self):
        self.gmp.create_note('foo', nvt_oid='oid1', seconds_active=0)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<active>0</active>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', seconds_active=-1)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<active>-1</active>'
            '</create_note>'
        )

        self.gmp.create_note('foo', nvt_oid='oid1', seconds_active=3600)

        self.connection.send.has_been_called_with(
            '<create_note>'
            '<text>foo</text>'
            '<nvt oid="oid1"/>'
            '<active>3600</active>'
            '</create_note>'
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from gvm.errors import RequiredArgument


class GmpModifyReportFormatTestCase:
    def test_modify_report_format(self):
        self.gmp.modify_report_format(report_format_id='rf1')

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1"/>'
        )

    def test_modify_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format(report_format_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format(report_format_id='')

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format('')

    def test_modify_report_format_with_summary(self):
        self.gmp.modify_report_format(report_format_id='rf1', summary='foo')

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<summary>foo</summary>'
            '</modify_report_format>'
        )

    def test_modify_report_format_with_name(self):
        self.gmp.modify_report_format(report_format_id='rf1', name='foo')

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<name>foo</name>'
            '</modify_report_format>'
        )

    def test_modify_report_format_with_active(self):
        self.gmp.modify_report_format(report_format_id='rf1', active=True)

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<active>1</active>'
            '</modify_report_format>'
        )

        self.gmp.modify_report_format(report_format_id='rf1', active=False)

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<active>0</active>'
            '</modify_report_format>'
        )

    def test_modify_report_format_with_param_name(self):
        self.gmp.modify_report_format(report_format_id='rf1', param_name='foo')

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<param>'
            '<name>foo</name>'
            '</param>'
            '</modify_report_format>'
        )

    def test_modify_report_format_with_param_name_and_value(self):
        self.gmp.modify_report_format(
            report_format_id='rf1', param_name='foo', param_value='bar'
        )

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<param>'
            '<name>foo</name>'
            '<value>bar</value>'
            '</param>'
            '</modify_report_format>'
        )

        self.gmp.modify_report_format(
            report_format_id='rf1', param_name='foo', param_value=''
        )

        self.connection.send.has_been_called_with(
            '<modify_report_format report_format_id="rf1">'
            '<param>'
            '<name>foo</name>'
            '<value></value>'
            '</param>'
            '</modify_report_format>'
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import os
import socketserver
import tempfile
import threading
import uuid

from gvm.connections import UnixSocketConnection, DEFAULT_TIMEOUT
from gvm.errors import GvmError


class DummyRequestHandler(socketserver.BaseRequestHandler):
    def handle(self):
        response = bytes(
            "<gmp_response status=\"200\" status_text=\"OK\"/>", 'utf-8'
        )
        self.request.sendall(response)


class ThreadedUnixStreamServer(
    socketserver.ThreadingMixIn, socketserver.UnixStreamServer
):
    pass


class UnixSocketConnectionTestCase(unittest.TestCase):
    def setUp(self):
        self.socketname = "%s/%s.sock" % (
            tempfile.gettempdir(),
            str(uuid.uuid4()),
        )
        self.sockserv = ThreadedUnixStreamServer(
            self.socketname, DummyRequestHandler
        )
        self.server_thread = threading.Thread(
            target=self.sockserv.serve_forever
        )
        self.server_thread.daemon = True
        self.server_thread.start()

    def tearDown(self):
        self.sockserv.shutdown()
        self.sockserv.server_close()
        os.unlink(self.socketname)

    def test_unix_socket_connection_connect_read(self):
        self.connection = UnixSocketConnection(
            path=self.socketname, timeout=DEFAULT_TIMEOUT
        )
        self.connection.connect()
        self.connection.read()
        self.connection.disconnect()

    def test_unix_socket_connection_connect_send_bytes_read(self):
        self.connection = UnixSocketConnection(
            path=self.socketname, timeout=DEFAULT_TIMEOUT
        )
        self.connection.connect()
        self.connection.send(bytes("<gmp/>", 'utf-8'))
        self.connection.read()
        self.connection.disconnect()

    def test_unix_socket_connection_connect_send_str_read(self):
        self.connection = UnixSocketConnection(
            path=self.socketname, timeout=DEFAULT_TIMEOUT
        )
        self.connection.connect()
        self.connection.send("<gmp/>")
        self.connection.read()
        self.connection.disconnect()

    def test_unix_socket_send_unconnected_socket(self):
        self.connection = UnixSocketConnection(
            path=self.socketname, timeout=DEFAULT_TIMEOUT
        )
        with self.assertRaises(GvmError):
            self.connection.send("<gmp>/")


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmpv208 import TimeUnit, get_time_unit_from_string


class GetTimeUnitFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            get_time_unit_from_string('foo')

    def test_none_or_empty(self):
        ct = get_time_unit_from_string(None)
        self.assertIsNone(ct)
        ct = get_time_unit_from_string('')
        self.assertIsNone(ct)

    def test_second(self):
        ct = get_time_unit_from_string('second')
        self.assertEqual(ct, TimeUnit.SECOND)

    def test_minute(self):
        ct = get_time_unit_from_string('minute')
        self.assertEqual(ct, TimeUnit.MINUTE)

    def test_hour(self):
        ct = get_time_unit_from_string('hour')
        self.assertEqual(ct, TimeUnit.HOUR)

    def test_day(self):
        ct = get_time_unit_from_string('day')
        self.assertEqual(ct, TimeUnit.DAY)

    def test_week(self):
        ct = get_time_unit_from_string('week')
        self.assertEqual(ct, TimeUnit.WEEK)

    def test_month(self):
        ct = get_time_unit_from_string('month')
        self.assertEqual(ct, TimeUnit.MONTH)

    def test_year(self):
        ct = get_time_unit_from_string('year')
        self.assertEqual(ct, TimeUnit.YEAR)

    def test_decade(self):
        ct = get_time_unit_from_string('decade')
        self.assertEqual(ct, TimeUnit.DECADE)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from gvm.errors import RequiredArgument


class GmpGetTagTestCase:
    def test_get_tag(self):
        self.gmp.get_tag('t1')

        self.connection.send.has_been_called_with('<get_tags tag_id="t1"/>')

        self.gmp.get_tag(tag_id='t1')

        self.connection.send.has_been_called_with('<get_tags tag_id="t1"/>')

    def test_get_tag_missing_tag_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_tag(tag_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_tag('')


if __name__ == '__main__':
    unittest.main()
